SystemOrganization addCategory: #'CS528su13AES-Payroll'!

SortedCollection variableSubclass: #SortedSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!SortedSequence commentStamp: '<historical>' prior: 0!
I add one method to SortedCollection.  It would be just as easy to add the method
directly to the class instead of making a subclass, but this makes it easier to see
the new method.

The method is indexOfStartOfIntervalContaining: anElement
It returns the index of anElement or, if it is not present, the index of the
largest element smaller than it.  It is useful when you want to have a
table that is sorted by keys and you want to look up in the table to
find the nearest match to a key.!

!SortedSequence methodsFor: 'accessing' stamp: ''!
indexOfStartOfIntervalContaining: anElement 
	"Return the index of anElement or, if it is not present, of the index of the
	largest element smaller than it."

	self isEmpty ifTrue: [^0].
	^(self indexForInserting: anElement) - firstIndex
			! !

TestCase subclass: #EmployeeTestCase
	instanceVariableNames: 'susan day1 normalMonth aLittleOvertime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!EmployeeTestCase methodsFor: 'Running' stamp: 'dd 1/11/2005 19:36'!
setUp
	day1 := Date 
				newDay: 11
				month: 7
				year: 2001.
	susan := Employee new named: 'Susan'.
	normalMonth := #(#(40 0) #(40 0) #(40 0) #(40 0)).
	aLittleOvertime := #(#(40 0) #(50 0) #(40 0) #(40 0))
			! !

!EmployeeTestCase methodsFor: 'Testing' stamp: 'EricShahkarami 6/22/2013 18:22'!
testChangeSalary
	susan changeSalaryFor: day1 to: 20.
	self assert: (susan salaryAt: day1) = 20.
	day1 := day1 addDays: 7.
	susan changeSalaryFor: day1 to: 25.
	self assert: (susan salaryAt: (day1 addDays: 28)) = 25
			! !

TestCase subclass: #PayrollSystemTestCase
	instanceVariableNames: 'myPayroll day1 susan george normalMonth aLittleOvertime twoWeeks dayJohnStarted john dayJohnFirstPaycheck'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!PayrollSystemTestCase methodsFor: 'private' stamp: 'EricShahkarami 6/22/2013 18:31'!
postTimeCards: collection at: aDate to: anEmployee 
	| timecardDay |
	timecardDay := aDate.
	collection do: 
			[:each | 
			anEmployee 
				postTimeCardFor: timecardDay
				hoursWorked: each first
				vacation: each last.
			timecardDay := timecardDay addDays: 7]
			! !

!PayrollSystemTestCase methodsFor: 'private' stamp: 'EricShahkarami 6/22/2013 18:31'!
postTimeCards: collection to: anEmployee 
	| timecardDay |
	timecardDay := dayJohnStarted. "day1."
	collection do: 
			[:each | 
			anEmployee 
				postTimeCardFor: timecardDay
				hoursWorked: each first
				vacation: each last.
			timecardDay := timecardDay addDays: 7]
			! !

!PayrollSystemTestCase methodsFor: 'Running' stamp: 'EricShahkarami 6/19/2013 12:06'!
setUp
	myPayroll := PayrollSystem new name: 'ACME'.
	day1 := Date newDay: 11 month: 7 year: 2001. 
	dayJohnStarted := Date newDay: 1 month: 5 year: 1999.
	dayJohnFirstPaycheck := Date newDay: 15 month: 5 year: 1999.
	susan := Employee new named: 'Susan'.
	susan changeSalaryFor: day1 to: 20.
	george := Employee new named: 'George'.
	george changeSalaryFor: day1 to: 12.
	john := Employee new named: 'John Doe'.
	john changeSalaryFor:  dayJohnStarted to: 20.
	twoWeeks := #(#(40 0) #(40 0) #(0 0) #(0 0)).
	normalMonth := #(#(40 0) #(40 0) #(40 0) #(40 0)).
	aLittleOvertime := #(#(40 0) #(50 0) #(40 0) #(40 0))
	! !

!PayrollSystemTestCase methodsFor: 'testing' stamp: ''!
testAddEmployee
	myPayroll addEmployee: susan.
	self assert: (myPayroll employeeNames includes: 'Susan').
	myPayroll addEmployee: george.
	self assert: (myPayroll employeeNamed: 'Susan') = susan
			! !

!PayrollSystemTestCase methodsFor: 'testing' stamp: 'EricShahkarami 6/22/2013 18:32'!
testEarnings
	myPayroll
		addEmployee: susan;
		addEmployee: george.
	self postTimeCards: normalMonth at: day1 to: susan.
	self postTimeCards: aLittleOvertime at: day1 to: george.
	self assert: (susan earnedAt: (day1 addDays: 28)) = 3200.
	self assert: (george earnedAt: (day1 addDays: 28)) = 2100
			! !

!PayrollSystemTestCase methodsFor: 'testing' stamp: 'EricShahkarami 6/22/2013 18:33'!
testTakeHomePay
	| johnPaycheck |

	myPayroll addEmployee: john.
	self postTimeCards: twoWeeks at: dayJohnStarted to: john.
	johnPaycheck := john makePaycheckAt: dayJohnFirstPaycheck.
	Transcript show: (johnPaycheck amountPaid).
	self assert: (johnPaycheck amountPaid) = 1188.8.! !

TestCase subclass: #PayrollVacationsTestCase
	instanceVariableNames: 'myPayroll day1 susan george normalMonth monthWithVacation aLittleOvertime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!PayrollVacationsTestCase methodsFor: 'private' stamp: 'EricShahkarami 6/22/2013 18:35'!
postTimeCards: collection at: aDate to: anEmployee 
	| timecardDay |
	timecardDay := aDate.
	collection do: 
			[:each | 
			anEmployee 
				postTimeCardFor: timecardDay
				hoursWorked: each first
				vacation: each last.
			timecardDay := timecardDay addDays: 7]
			! !

!PayrollVacationsTestCase methodsFor: 'running' stamp: 'EricShahkarami 6/19/2013 11:55'!
setUp
	myPayroll := PayrollSystem new name: 'ACME'.
	day1 := Date newDay: 11 month: 7 year: 2001. 
	susan := Employee new named: 'Susan'.
	susan changeSalaryFor: day1 to: 20.
	george := Employee new named: 'George'.
	george changeSalaryFor: day1 to: 12.
	normalMonth := #(#(40 0) #(40 0) #(40 0) #(40 0)).
	monthWithVacation := #(#(40 1.2) #(40 0) #(40 2) #(40 0)).
	aLittleOvertime := #(#(40 0) #(50 0) #(40 0) #(40 0))
	! !

!PayrollVacationsTestCase methodsFor: 'testing' stamp: 'EricShahkarami 6/22/2013 18:35'!
testAccruedVacationOverMonthWithVacation
	"Normal 40 hour work week with 3.2 hours of vacation. Susan has salary of $20/hr and George a salary of $12/hr.
	Though salary should not affect accrued vacation."

	myPayroll 
		addEmployee: susan;
		addEmployee: george.
	self postTimeCards: monthWithVacation at: day1 to: susan.
	self postTimeCards: monthWithVacation at: day1 to: george.
	self assert: (susan accruedVacationAt: (day1 addDays: 28)) = 3.2.
	self assert: (susan accruedVacationAt: (day1 addDays: 28)) = 3.2! !

!PayrollVacationsTestCase methodsFor: 'testing' stamp: 'EricShahkarami 6/22/2013 18:36'!
testAccruedVacationOverNormalMonth
	"Normal 40 hour work week. Susan has salary of $20/hr and George a salary of $12/hr.
	Though salary should not affect accrued vacation."

	myPayroll 
		addEmployee: susan;
		addEmployee: george.
	self postTimeCards: normalMonth at: day1 to: susan.
	self postTimeCards: normalMonth at: day1 to: george.
	self assert: (susan accruedVacationAt: (day1 addDays: 28)) = 6.4.
	self assert: (susan accruedVacationAt: (day1 addDays: 28)) = 6.4
! !

!PayrollVacationsTestCase methodsFor: 'testing' stamp: 'EricShahkarami 6/22/2013 18:36'!
testAccruedVacationWithOvertime
	"Normal 40 hour work week with one 50 hour day of overtime. Susan has salary of $20/hr and George a salary of $12/hr.
	Though salary should not affect accrued vacation."

	myPayroll 
		addEmployee: susan;
		addEmployee: george.
	self postTimeCards:  aLittleOvertime at: day1 to: susan.
	self postTimeCards:  aLittleOvertime at: day1 to: george.
	self assert: (susan accruedVacationAt: (day1 addDays: 28)) = 6.4.
	self assert: (susan accruedVacationAt: (day1 addDays: 28))= 6.4
! !

TestCase subclass: #SortedSequenceTestCase
	instanceVariableNames: 'sequence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!SortedSequenceTestCase methodsFor: 'Running' stamp: ''!
setUp
	sequence := SortedSequence new.
	sequence add: 1; add: 80; add: 500; add: 501
			! !

!SortedSequenceTestCase methodsFor: 'Testing' stamp: ''!
testIndex
	"(SortedSequenceTestCase selector: #testIndex) run"

	self assert: (sequence indexOfStartOfIntervalContaining: 1) = 1.
	self assert: (sequence indexOfStartOfIntervalContaining: 79) = 1.
	self assert: (sequence indexOfStartOfIntervalContaining: 80) = 2.
	self assert: (sequence indexOfStartOfIntervalContaining: 501)	= 4
			! !

Object subclass: #Employee
	instanceVariableNames: 'name transactions salary earned paid withholding accruedVacation taxRule'
	classVariableNames: 'HeadTaxRate MarriedSeparateTaxRate MarriedTaxRate SingleTaxRate'
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!Employee commentStamp: '<historical>' prior: 0!
I represent an employee in a payroll system.  Thus, I keep track of the number of hours worked,
the wages paid, the vacation accrued, the taxes withheld, and so on.  My values change only
when transactions are posted to me.  Transactions are subclasses of EmployeeTransaction.

Variables
	name 				<String>
	transactions 			<Collection of Transactions, sorted by date>
	salary 				<ValueWithHistory>
	earned  				<ValueWithHistory>
	paid  				<ValueWithHistory>
	withholding  			<ValueWithHistory>
	accruedVacation  	<ValueWithHistory>
	taxRule 				<TaxRule>!

!Employee class methodsFor: 'class initialization' stamp: ''!
initialize
	"Employee initialize"
	SingleTaxRate := TaxRule new.
	SingleTaxRate
		breakPoint: 0 rate: 0.15;
		breakPoint: 22750 rate: 0.28;
		breakPoint: 55100 rate: 0.31;
		breakPoint: 115000 rate: 0.36;
		breakPoint: 250000 rate: 0.396.

	MarriedTaxRate := TaxRule new.
	MarriedTaxRate
		breakPoint: 0 rate: 0.15;
		breakPoint: 38000 rate: 0.28;
		breakPoint: 91850 rate: 0.31;
		breakPoint: 140000 rate: 0.36;
		breakPoint: 250000 rate: 0.396.

	MarriedSeparateTaxRate := TaxRule new.
	MarriedSeparateTaxRate
		breakPoint: 0 rate: 0.15;
		breakPoint: 19500 rate: 0.28;
		breakPoint: 45925 rate: 0.31;
		breakPoint: 70000 rate: 0.36;
		breakPoint: 125000 rate: 0.396.

	HeadTaxRate := TaxRule new.
	HeadTaxRate
		breakPoint: 0 rate: 0.15;
		breakPoint: 30500 rate: 0.28;
		breakPoint: 78700 rate: 0.31;
		breakPoint: 127500 rate: 0.36;
		breakPoint: 250000 rate: 0.396.
			! !

!Employee methodsFor: 'accessing' stamp: 'EricShahkarami 6/22/2013 17:45'!
accruedVacationAt: aDate
	^(accruedVacation at: aDate) asFloat ! !

!Employee methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 18:22'!
accruedVacationAt: aDate add: anAmount
	accruedVacation at: aDate add: anAmount! !

!Employee methodsFor: 'actions' stamp: 'EricShahkarami 6/5/2013 20:05'!
changeSalaryFor: aDate to: dollarPerHour 
	self postTransaction: (SalaryChange
			date: aDate
			employee: self
			salary: dollarPerHour)
			! !

!Employee methodsFor: 'accessing' stamp: 'EricShahkarami 6/22/2013 17:45'!
earnedAt: aDate
	^(earned at: aDate)
			! !

!Employee methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 17:52'!
earnedAt: aDate add: anAmount
	earned at: aDate add: anAmount
			! !

!Employee methodsFor: 'private' stamp: ''!
federalTaxesOn: money assumingTotalIs: estimatedEarnings
	^money * (taxRule rateFor: estimatedEarnings)
			! !

!Employee methodsFor: 'actions' stamp: 'EricShahkarami 6/22/2013 17:44'!
makePaycheckAt: aDate 
	"Create a paycheck for the amount owed to the employee. It hasn't 
	been posted."

	| amountPaid amountWithheld |
	amountPaid := (earned at: aDate) - (paid at: aDate) - (withholding at: aDate).
	amountWithheld := self taxesOn: amountPaid at: aDate.
	amountPaid := amountPaid - amountWithheld.
	^Paycheck
		date: aDate
		employee: self
		paid: amountPaid
		withheld: amountWithheld
		totalPaid: (paid at: aDate) + amountPaid
		totalWithheld: (withholding at: aDate) + amountWithheld
			! !

!Employee methodsFor: 'accessing' stamp: ''!
name
	^name
			! !

!Employee methodsFor: 'initialize-release' stamp: 'EricShahkarami 6/22/2013 17:41'!
named: aString
	name := aString.
	transactions := OrderedCollection new.
	salary := ValueWithHistory zero.
	earned := ValueWithHistory zero.
	paid := ValueWithHistory zero.
	withholding := ValueWithHistory zero.
	accruedVacation := ValueWithHistory zero.
	taxRule := SingleTaxRate
			! !

!Employee methodsFor: 'accessing' stamp: 'EricShahkarami 6/22/2013 17:46'!
paidAt: aDate
	^(paid at: aDate)
			! !

!Employee methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 17:53'!
paidAt: aDate add: anAmount
	paid at: aDate add: anAmount
			! !

!Employee methodsFor: 'actions' stamp: 'EricShahkarami 6/5/2013 20:15'!
postTimeCardFor: aDate hoursWorked: hoursW vacation: hoursV 
	self postTransaction: (Timecard
			date: aDate
			employee: self
			worked: hoursW
			vacation: hoursV)
			! !

!Employee methodsFor: 'posting' stamp: ''!
postTransaction: aTransaction
	"Public interface to record and process (i.e. post) a transaction."
	aTransaction postTo: self.
	transactions add: aTransaction
			! !

!Employee methodsFor: 'accessing' stamp: ''!
salary: anAmount
	salary := anAmount
			! !

!Employee methodsFor: 'accessing' stamp: 'EricShahkarami 6/22/2013 17:47'!
salaryAt: aDate
	^(salary at: aDate)
			! !

!Employee methodsFor: 'accessing' stamp: 'EricShahkarami 6/22/2013 17:56'!
salaryStarting: aDate become: anAmount
	salary starting: aDate become: anAmount
			! !

!Employee methodsFor: 'private' stamp: 'EricShahkarami 6/22/2013 18:25'!
taxesOn: money at: date 
	"Return the taxes to be withheld from the money earned."
	"This is just a guess - I didn't look up the tax rules."

	| socialSecurity federalTaxes stateTaxes estimatedEarnings |
	socialSecurity := ( 0 max: (money min: 60600 - (earned at: date)) )
				* 0.077.
	estimatedEarnings := (earned at: date) + money * date day / date daysInYear.
	federalTaxes := self federalTaxesOn: money assumingTotalIs: estimatedEarnings.
	stateTaxes := money * 0.03.	"Illinois has a flat tax."
	^socialSecurity + federalTaxes + stateTaxes
			! !

!Employee methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 17:54'!
vacationAt: aDate add: anAmount
	accruedVacation at: aDate add: anAmount! !

!Employee methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 17:54'!
withholdingAt: aDate add: anAmount 
	withholding at: aDate add: anAmount
			! !

Object subclass: #EmployeeTransaction
	instanceVariableNames: 'date employee'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!EmployeeTransaction commentStamp: '<historical>' prior: 0!
I'm an abstract class of transactions that can be posted to an Employee.

My most important method is postTo:, which is implemented by subclasses
to send a message to the Employee to post the transaction.  The Employee
has all the information, so it has to be the one to implement posting, but the
postTo: method implements a standard interface so that clients of the
payroll system can always post a transaction by sending the postTransaction:
message to an Employee.

Variables
	date		<Date>   Date at which transaction occured.
	employee	<Employee>

!

!EmployeeTransaction methodsFor: 'accessing' stamp: ''!
date
	^date
			! !

!EmployeeTransaction methodsFor: 'posting' stamp: ''!
post
	employee postTransaction: self
			! !

!EmployeeTransaction methodsFor: 'posting' stamp: ''!
postTo: anEmployee
	"double dispatching"
	self subclassResponsibility
			! !

EmployeeTransaction subclass: #Paycheck
	instanceVariableNames: 'amountPaid taxes totalPaid totalTaxes'
	classVariableNames: 'AmountFormat DateFormat'
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!Paycheck commentStamp: '<historical>' prior: 0!
Transaction representing a payment to an employee.!

!Paycheck class methodsFor: 'class initialization' stamp: 'EricShahkarami 6/16/2013 19:10'!
date: aDate employee: anEmployee paid: amount1 withheld: amount2 totalPaid: amount3 totalWithheld: amount4
	^ (self new
	date: aDate 
	employee: anEmployee 
	paid: amount1 
	withheld: amount2 
	totalPaid: amount3 
	totalWithheld: amount4).! !

!Paycheck class methodsFor: 'class initialization' stamp: 'dd 12/23/2004 18:15'!
initialize
	"Paycheck initialize"

	"AmountFormat := NumberPrintPolicy newFor: #usCurrency.
	DateFormat := TimestampPrintPolicy newFor: #us"
			! !

!Paycheck methodsFor: 'accessing' stamp: ''!
amountPaid
	^amountPaid
			! !

!Paycheck methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee paid: amount1 withheld: amount2 totalPaid: amount3 totalWithheld: amount4
	date := aDate.
	employee := anEmployee.
	amountPaid := amount1.
	taxes := amount2.
	totalPaid := amount3.
	totalTaxes := amount4
			! !

!Paycheck methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 17:59'!
postTo: anEmployee
	anEmployee paidAt: date add: amountPaid.
	anEmployee withholdingAt: date add: taxes. 

	"anEmployee incrementPaid: amountPaid.
	anEmployee incrementWithholding: taxes"
			! !

!Paycheck methodsFor: 'printing' stamp: 'dd 12/23/2004 19:31'!
printOnCheckStream: aStream
	"This method is supposed to print information on a check that is on the printer, and
	aStream is the printer.  Unfortunately, the whole method is bogus, because we don't
	know what the printer or the check are like.  I'm assuming that the check has two-parts.
	One is the actual check, and we need to print the employee's name and the amount.
	The second is the check stub, which will say how much is being paid and how much
 	is being withheld. I'm also assuming that the check is preprinted with the name of the
	company, and everything else that is usual for a check."

	aStream cr; cr.
	aStream next: 40 put: (Character space).
	"DateFormat print: date on: aStream."
	date printOn: aStream.
	aStream cr.
	aStream nextPutAll: ((String new: 40 withAll: Character space) copyReplaceFrom: 1 to: employee name size with: employee name).
	"AmountFormat print: amountPaid on: aStream."
	amountPaid printOn: aStream.
	aStream cr; cr; cr; cr.
	aStream nextPutAll: 'this month  '.
	"AmountFormat print: amountPaid on: aStream.
	AmountFormat print: taxes on: aStream."
	aStream nextPutAll: ' amountPaid: $'.
	amountPaid printOn: aStream.
	aStream nextPutAll: ' taxes: $'.
	taxes printOn: aStream.
	aStream cr.
	aStream nextPutAll: 'year to date '.
	"AmountFormat print: totalPaid on: aStream.
	AmountFormat print: totalTaxes on: aStream."
	aStream nextPutAll: ' amountPaid: $'.
	totalPaid printOn: aStream.
	aStream nextPutAll: ' taxes: $'.
	totalTaxes printOn: aStream.
	aStream cr; cr.
			! !

!Paycheck methodsFor: 'accessing' stamp: ''!
taxes
	^taxes
			! !

EmployeeTransaction subclass: #SalaryChange
	instanceVariableNames: 'newSalary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!SalaryChange commentStamp: '<historical>' prior: 0!
Transaction representing a change in salary, usually a raise.!

!SalaryChange class methodsFor: 'class initialization' stamp: 'EricShahkarami 6/16/2013 19:11'!
date: aDate employee: anEmployee salary: dollarPerHour
	^ (self new 
	date: aDate 
	employee: anEmployee 
	salary: dollarPerHour).
			! !

!SalaryChange methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee salary: dollarPerHour
	date := aDate.
	employee := anEmployee.
	newSalary := dollarPerHour
			! !

!SalaryChange methodsFor: 'accessing' stamp: ''!
newSalary
	^newSalary
			! !

!SalaryChange methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 18:00'!
postTo: anEmployee
	anEmployee salaryStarting: date become: newSalary. 
	
	"anEmployee salary: newSalary"
			! !

EmployeeTransaction subclass: #Timecard
	instanceVariableNames: 'hoursWorked hoursVacation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!Timecard commentStamp: '<historical>' prior: 0!
Transaction representing the work that the employee has done.!

!Timecard class methodsFor: 'class initialization' stamp: 'EricShahkarami 6/16/2013 19:19'!
date: aDate employee: anEmployee worked: hoursW vacation: hoursV
	^ (self new date: aDate employee: anEmployee worked: hoursW vacation: hoursV).! !

!Timecard methodsFor: 'initialize-release' stamp: ''!
date: aDate employee: anEmployee worked: hoursW vacation: hoursV
	date := aDate.
	employee := anEmployee.
	hoursWorked := hoursW.
	hoursVacation := hoursV
			! !

!Timecard methodsFor: 'accessing' stamp: ''!
hoursVacation
	^hoursVacation
			! !

!Timecard methodsFor: 'accessing' stamp: ''!
hoursWorked
	^hoursWorked
			! !

!Timecard methodsFor: 'posting' stamp: 'EricShahkarami 6/22/2013 18:21'!
postTo: anEmployee 
	| money overtime nonovertime |
	overtime := hoursWorked - 40 max: 0.
	nonovertime := hoursWorked min: 40.
	money := overtime * 1.5 + nonovertime * (anEmployee salaryAt: date).
	anEmployee earnedAt: date add: money.
	anEmployee accruedVacationAt: date add: (nonovertime / 25) - hoursVacation
	
	"anEmployee incrementEarned: money.
	anEmployee incrementVacation: (nonovertime / 25) - hoursVacation"! !

Object subclass: #PayrollSystem
	instanceVariableNames: 'employees paychecks companyName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!PayrollSystem commentStamp: '<historical>' prior: 0!
I am the top-level of the payroll system.  I keep track of the employees
and can "run the payroll", i.e. write paychecks for the employees.  To
run the payroll, send the messages
	makePaychecksAt: paydate	- to create the paycheck transactions
	printPaychecksOn: aStream	- to print them out 
	postPaychecks				- to record them in employees

Note that the employees are not marked with having been paid until
the paychecks are posted, so just creating the transactions or even
printing them is not enough.

Variables
	employees   <Dictionary from String to Employee>
	paychecks 	<Collection of Paycheck>	holds paychecks that have been created until they have been posted!

!PayrollSystem class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize
			! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
addEmployee: anEmployee
	employees at: anEmployee name put: anEmployee
			! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
employeeNamed: aName
	^employees at: aName
			! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
employeeNames
	^employees keys asOrderedCollection
			! !

!PayrollSystem methodsFor: 'accessing' stamp: 'EricShahkarami 6/20/2013 17:59'!
employees
	^employees ! !

!PayrollSystem methodsFor: 'initialize-release' stamp: ''!
initialize
	employees := Dictionary new.
			! !

!PayrollSystem methodsFor: 'saving-loading' stamp: 'EricShahkarami 6/20/2013 18:07'!
loadFromDisk: filename 
	| loadedPayroll |
	
	loadedPayroll := FLMaterializer materializeFromFileNamed: filename. 
	companyName := loadedPayroll name.
	employees := loadedPayroll employees.
	paychecks := loadedPayroll paychecks
	! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
makePaychecksAt: aDate 
	paychecks := employees collect: [:eachEmployee | eachEmployee makePaycheckAt: aDate]
			! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
name
	^companyName
			! !

!PayrollSystem methodsFor: 'accessing' stamp: ''!
name: aString
	companyName := aString
			! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
paycheckString
	| aStream |
	aStream := WriteStream on: (String new: 20).
	self printPaychecksOn: aStream.
	^aStream contents
			! !

!PayrollSystem methodsFor: 'accessing' stamp: 'EricShahkarami 6/20/2013 17:59'!
paychecks
	^paychecks! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
postPaychecks
	paychecks do: [:each | each post].
	paychecks := #()
			! !

!PayrollSystem methodsFor: 'actions' stamp: ''!
printPaychecksOn: aStream 
	paychecks do: [:each | each printOnCheckStream: aStream]
			! !

!PayrollSystem methodsFor: 'saving-loading' stamp: 'EricShahkarami 6/20/2013 17:35'!
saveToDisk: filename 
	FLSerializer serialize: self toFileNamed: filename
! !

Object subclass: #TaxRule
	instanceVariableNames: 'breakPoints rates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-Payroll'!

!TaxRule commentStamp: '<historical>' prior: 0!
I represent a tax table.  Given a yearly taxable income, I can give the tax rate for it.
I store the tax table as a sequence of amounts, and the tax rate for that amount.
For example, in 1994 the tax rate for a single person (schedule X) was
           $0		15%
  $22750	28%
  $55100	31%
$115000	36%
$250000	39.6%

breakpoints  	<SortedCollection of: integer>
rates	 	<OrderedCollection of: integer>!

!TaxRule class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize
			! !

!TaxRule methodsFor: 'accessing' stamp: ''!
breakPoint: aNumber rate: anotherNumber 
	"Add an entry to the tax table"

	| index |
	index := breakPoints indexOfStartOfIntervalContaining: aNumber.
	breakPoints add: aNumber.
	rates add: anotherNumber beforeIndex: index + 1
			! !

!TaxRule methodsFor: 'initialize-release' stamp: ''!
initialize
	breakPoints := SortedSequence new.
	rates := OrderedCollection new.
			! !

!TaxRule methodsFor: 'accessing' stamp: ''!
rateFor: aNumber 
	"Return the tax rate for someone whose taxable income is 
	aNumber dollars."

	^rates at: (breakPoints indexOfStartOfIntervalContaining: aNumber)
			! !
Paycheck initialize!
Employee initialize!
